try:
    import appium
    appium_installed = True
except ImportError:
    appium_installed = False

from .webdriver import WebDriver, WebDriverType, Browser, BrowserType
from .staticelement import StaticElement
from .identifier import Identifier
from .waiter import Waiter
from .exceptions import EasyiumException, TimeoutException, NoSuchElementException, NotPersistException, \
    LatePersistException, UnsupportedWebDriverTypeException, InvalidByException, UnsupportedOperationForWebDriver
from .config import default_config

__author__ = 'karl.gong'
